using Android.Widget;
using Android.Graphics;
using System;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using ListViewSample.Droid;
using ListViewSample;
using Xamarin.Forms;
using Xamarin.Forms.Platform.Android;

[assembly: ExportRenderer(typeof(SfImageView), typeof(SfImageViewRenderer))]
namespace ListViewSample.Droid
{
    internal class SfImageViewRenderer : ImageRenderer
    {
        public SfImageView FormsView
        {
            get { return (SfImageView)this.Element; }
        }

        protected override void OnElementPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Source" || e.PropertyName == "Renderer")
            {
                if (this.FormsView.Source is StreamImageSource)
                    updateNativeImage();
            }
            else
                base.OnElementPropertyChanged(sender, e);
        }

        private void updateNativeImage()
        {
            var stream = GetStreamFromImageSourceAsync((StreamImageSource)this.FormsView.Source).GetAwaiter().GetResult();
            using (MemoryStream memoryStream = new MemoryStream())
            {
                stream.CopyTo(memoryStream);
                if (Control != null)
                    Control.SetImageBitmap(BitmapFactory.DecodeByteArray(memoryStream.ToArray(), 0, (int)stream.Length));
            }
        }

        private Task<Stream> GetStreamFromImageSourceAsync(StreamImageSource imageSource, CancellationToken cancellationToken = default(CancellationToken))
        {
            if (imageSource.Stream != null)
                return imageSource.Stream(cancellationToken);
            return null;
        }
    }
}